import { Injectable } from '@nestjs/common';
import * as moment from 'moment';
import { TransactionItemModel } from 'src/modules/transaction/transaction/data/models/transaction-item.model';
import { TicketDataService } from '../../data/services/ticket.service';
import { QueueOrder } from '../entities/order.entity';
import { QueueItem } from '../entities/queue-item.entity';
import { QueueTicket } from '../entities/ticket.entity';
import { TransactionModel } from 'src/modules/transaction/transaction/data/models/transaction.model';

@Injectable()
export class GenerateTransactionManager {
  constructor(private readonly ticketService: TicketDataService) {}

  async generate(transaction: TransactionModel) {
    const date = transaction.booking_date ?? transaction.invoice_date;
    const queue_date = moment(date, 'YYYY-MM-DD').unix();

    const { id, customer_name, customer_phone, invoice_code } = transaction;

    const customerOrder = {
      code: invoice_code,
      customer: customer_name,
      phone: customer_phone,
      date: queue_date * 1000,
      transaction_id: id,
    };

    const items = this.generateItems(transaction.items);

    const existTicket = await this.ticketService.ticketByUser(
      customer_name,
      customer_phone,
    );

    const insertTickets: QueueTicket[] = [];
    if (customer_name && customer_phone && existTicket) {
      existTicket.items.push(...items);
      await this.ticketService.updateQueueTicket(existTicket);

      return existTicket.order;
    } else {
      const ticket: QueueTicket = { ...customerOrder, items };
      const order: QueueOrder = { ...customerOrder, tickets: [ticket] };

      const queueOrder = await this.ticketService.createQueueOrder(order);
      insertTickets.push(...queueOrder.tickets);

      return queueOrder;
    }
  }

  generateItems(items: TransactionItemModel[]): QueueItem[] {
    const transactionItems = [];

    items.forEach((item) => {
      if (item.item.use_queue) {
        transactionItems.push({
          item_queue_id: item.item.item_queue_id,
          item_id: item.item_id,
          qty: item.qty,
        });
      }

      if (item.item.bundling_items.length > 0) {
        item.item.bundling_items.forEach((bundling_item) => {
          if (bundling_item.use_queue) {
            transactionItems.push({
              item_queue_id: bundling_item.item_queue_id,
              item_id: bundling_item.id,
              qty: item.qty,
            });
          }
        });
      }
    });

    return this.mergeQueueItems(transactionItems);
  }

  mergeQueueItems(arr) {
    const result = {};
    arr.forEach((item) => {
      if (result[item.item_id]) {
        result[item.item_id].qty += item.qty;
      } else {
        result[item.item_id] = { ...item };
      }
    });
    return Object.values<QueueItem>(result);
  }
}
